<?php

namespace App\Models\Product;

use App\Models\Base\ProductType;
use App\Traits\NextKey;
use App\Traits\OrderServiceOptions;

class Voip extends ProductType
{
	use NextKey;
	const RECORD_ID = '';

	use OrderServiceOptions;

	protected $order_attributes = [
		'options.phonenumber'=>[
			'request'=>'options.phonenumber',
			'key'=>'service_number',
			'validation'=>'nullable|size:10|unique:ab_service__voip,service_number',
			'validation_message'=>'Phone Number is a required field.',
		],
		'options.supplier'=>[
			'request'=>'options.supplier',
			'key'=>'order_info.supplier',
			'validation'=>'required_with:options.phonenumber',
			'validation_message'=>'Phone Supplier is a required field.',
		],
		'options.supplieraccnum'=>[
			'request'=>'options.supplieraccnum',
			'key'=>'order_info.supplieraccnum',
			'validation'=>'required_with:options.phonenumber',
			'validation_message'=>'Phone Supplier Account Number is a required field.',
		],
		'options.notes'=>[
			'request'=>'options.notes',
			'key'=>'order_info.notes',
			'validation'=>'required_if:options.phonenumber,null',
			'validation_message'=>'Special Instructions here.',
		],
	];

	protected $order_model = \App\Models\Service\Voip::class;

	public function getContractTermAttribute(): int
	{
		return 12;
	}

	public function getTypeAttribute()
	{
		return 'VOIP';
	}

	public function hasUsage(): bool
	{
		return TRUE;
	}
}